==> mule-netty-http-service-0.2.11-20251017.030835-22-test-sources.jar: org/mule/service/http/test/netty/utils/server/AuthenticationTestServer.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 */
package org.mule.service.http.test.netty.utils.server;

import static org.mule.runtime.core.api.util.FileUtils.getResourcePath;

import static com.google.common.collect.Multimaps.newMultimap;

import org.mule.runtime.api.util.CaseInsensitiveMapWrapper;
import org.mule.service.http.test.netty.utils.NtlmLoginAuthenticator;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Base64;
import java.util.Enumeration;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.google.common.collect.Multimap;
import com.google.common.collect.Sets;
import org.eclipse.jetty.security.ConstraintMapping;
import org.eclipse.jetty.security.ConstraintSecurityHandler;
import org.eclipse.jetty.security.HashLoginService;
import org.eclipse.jetty.security.LoginService;
import org.eclipse.jetty.security.authentication.BasicAuthenticator;
import org.eclipse.jetty.security.authentication.DigestAuthenticator;
import org.eclipse.jetty.server.Handler;
import org.eclipse.jetty.server.HttpInputOverHTTP;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.AbstractHandler;
import org.eclipse.jetty.server.handler.ContextHandler;
import org.eclipse.jetty.server.handler.ContextHandlerCollection;
import org.eclipse.jetty.util.security.Constraint;
import org.junit.rules.ExternalResource;

public class AuthenticationTestServer extends ExternalResource {

  public int httpPort;
  public static final String DEFAULT_RESPONSE = "{\"key1\":\"value1\",\"key2\":\"value2\"}";
  protected volatile Server server;

  protected volatile String method;
  protected volatile String uri;
  protected volatile Multimap<String, String> headers = newMultimap(new CaseInsensitiveMapWrapper<>(), Sets::newHashSet);

  protected volatile String body;

  public AuthenticationTestServer(int port) {
    this.httpPort = port;
  }

  @Override
  public void before() throws Exception {
    server = createServer();
    AbstractHandler handler = new TestHandler();
    String realmPath = getResourcePath("auth/realm.properties", getClass());
    server.setHandler(createAuthHandler(server, handler, realmPath));
    server.start();
  }

  private AbstractHandler createAuthHandler(Server server, AbstractHandler handler, String realmPath) {
    LoginService loginService = new HashLoginService("TestRealm", realmPath);
    server.addBean(loginService);

    ContextHandler basicContext = createBasicContextHandler(handler);
    ContextHandler digestContext = createDigestContextHandler(handler);
    ContextHandler ntlmContext = createNtlmContextHandler(handler);
    ContextHandler multiAuthContext = createMultiAuthContextHandler(handler);

    ContextHandlerCollection handlers = new ContextHandlerCollection();
    handlers.setHandlers(new Handler[] {basicContext, digestContext, ntlmContext, multiAuthContext});
    return handlers;
  }

  private ContextHandler createDigestContextHandler(AbstractHandler handler) {
    Constraint digestConstraint = new Constraint();
    digestConstraint.setName("auth");
    digestConstraint.setRoles(new String[] {"user"});
    digestConstraint.setAuthenticate(true);

    ConstraintMapping digestConstraintMapping = new ConstraintMapping();
    digestConstraintMapping.setConstraint(digestConstraint);
    digestConstraintMapping.setPathSpec("/*");

    ConstraintSecurityHandler digestSecurityHandler = new ConstraintSecurityHandler();
    digestSecurityHandler.setAuthenticator(new DigestAuthenticator());
    digestSecurityHandler.setConstraintMappings(new ConstraintMapping[] {digestConstraintMapping});
    digestSecurityHandler.setHandler(handler);

    ContextHandler digestContext = new ContextHandler("/digest");
    digestContext.setHandler(digestSecurityHandler);

    return digestContext;
  }

  protected ContextHandler createBasicContextHandler(AbstractHandler handler) {
    Constraint basicConstraint = new Constraint();
    basicConstraint.setName("auth");
    basicConstraint.setRoles(new String[] {"user"});
    basicConstraint.setAuthenticate(true);

    ConstraintMapping basicConstraintMapping = new ConstraintMapping();
    basicConstraintMapping.setConstraint(basicConstraint);
    basicConstraintMapping.setPathSpec("/*");

    ConstraintSecurityHandler basicSecurityHandler = new ConstraintSecurityHandler();
    basicSecurityHandler.setAuthenticator(new BasicAuthenticator());
    basicSecurityHandler.setConstraintMappings(new ConstraintMapping[] {basicConstraintMapping});
    basicSecurityHandler.setHandler(handler);

    ContextHandler basicContext = new ContextHandler("/basic");
    basicContext.setHandler(basicSecurityHandler);

    return basicContext;
  }

  private ContextHandler createNtlmContextHandler(AbstractHandler handler) {
    Constraint ntlmConstraint = new Constraint();
    ntlmConstraint.setName("auth");
    ntlmConstraint.setRoles(new String[] {"user"});
    ntlmConstraint.setAuthenticate(true);

    ConstraintMapping ntlmConstraintMapping = new ConstraintMapping();
    ntlmConstraintMapping.setConstraint(ntlmConstraint);
    ntlmConstraintMapping.setPathSpec("/*");

    ConstraintSecurityHandler ntlmSecurityHandler = new ConstraintSecurityHandler();
    ntlmSecurityHandler.setAuthenticator(new NtlmLoginAuthenticator());
    ntlmSecurityHandler.setConstraintMappings(new ConstraintMapping[] {ntlmConstraintMapping});
    ntlmSecurityHandler.setHandler(handler);

    ContextHandler ntlmContext = new ContextHandler("/ntlm");
    ntlmContext.setHandler(ntlmSecurityHandler);

    return ntlmContext;
  }

  private ContextHandler createMultiAuthContextHandler(AbstractHandler handler) {

    Constraint basicConstraint = new Constraint();
    basicConstraint.setName("basicAuth");
    basicConstraint.setRoles(new String[] {"user"});
    basicConstraint.setAuthenticate(true);

    ConstraintMapping basicConstraintMapping = new ConstraintMapping();
    basicConstraintMapping.setConstraint(basicConstraint);
    basicConstraintMapping.setPathSpec("/multi-auth/*");

    ConstraintSecurityHandler basicSecurityHandler = new ConstraintSecurityHandler();
    basicSecurityHandler.setAuthenticator(new BasicAuthenticator());
    basicSecurityHandler.setConstraintMappings(new ConstraintMapping[] {basicConstraintMapping});

    Constraint digestConstraint = new Constraint();
    digestConstraint.setName("digestAuth");
    digestConstraint.setRoles(new String[] {"user"});
    digestConstraint.setAuthenticate(true);

    ConstraintMapping digestConstraintMapping = new ConstraintMapping();
    digestConstraintMapping.setConstraint(digestConstraint);
    digestConstraintMapping.setPathSpec("/multi-auth/*");

    ConstraintSecurityHandler digestSecurityHandler = new ConstraintSecurityHandler();
    digestSecurityHandler.setAuthenticator(new DigestAuthenticator());
    digestSecurityHandler.setConstraintMappings(new ConstraintMapping[] {digestConstraintMapping});

    Constraint ntlmConstraint = new Constraint();
    ntlmConstraint.setName("ntlmAuth");
    ntlmConstraint.setRoles(new String[] {"user"});
    ntlmConstraint.setAuthenticate(true);

    ConstraintMapping ntlmConstraintMapping = new ConstraintMapping();
    ntlmConstraintMapping.setConstraint(ntlmConstraint);
    ntlmConstraintMapping.setPathSpec("/multi-auth/*");

    ConstraintSecurityHandler ntlmSecurityHandler = new ConstraintSecurityHandler();
    ntlmSecurityHandler.setAuthenticator(new NtlmLoginAuthenticator());
    ntlmSecurityHandler.setConstraintMappings(new ConstraintMapping[] {ntlmConstraintMapping});

    ContextHandlerCollection securityHandlers = new ContextHandlerCollection();
    securityHandlers.setHandlers(new Handler[] {basicSecurityHandler, digestSecurityHandler, ntlmSecurityHandler});

    ContextHandler multiAuthContext = new ContextHandler("/multi-auth");
    multiAuthContext.setHandler(securityHandlers);

    multiAuthContext.setHandler(new ContextHandlerCollection() {

      @Override
      public void handle(String target, Request baseRequest, HttpServletRequest request, HttpServletResponse response)
          throws IOException, ServletException {
        securityHandlers.handle(target, baseRequest, request, response);
        if (!baseRequest.isHandled()) {
          handler.handle(target, baseRequest, request, response);
        }
      }
    });

    return multiAuthContext;
  }

  @Override
  public void after() {
    if (server != null) {
      try {
        server.stop();
      } catch (Exception e) {
        throw new RuntimeException(e);
      }
    }
  }

  protected Server createServer() {
    return new Server(httpPort);
  }

  public void addHandler(AbstractHandler handler) {
    this.server.setHandler(handler);
  }

  private class TestHandler extends AbstractHandler {

    @Override
    public void handle(String target, Request baseRequest, HttpServletRequest request, HttpServletResponse response)
        throws IOException {

      handleRequest(baseRequest, request, response);
      baseRequest.setHandled(true);
    }
  }

  protected void handleRequest(Request baseRequest, HttpServletRequest request, HttpServletResponse response) throws IOException {
    extractBaseRequestParts(baseRequest);

    if (!isAuthenticated(request)) {
      response.addHeader("WWW-Authenticate", "Basic realm=\"Test\"");
      response.addHeader("WWW-Authenticate", "NTLM");
      response
          .addHeader("WWW-Authenticate",
                     "Digest realm=\"Test\", qop=\"auth\", nonce=\"dcd98b7102dd2f0e8b11d0f600bfb0c093\", opaque=\"5ccc069c403ebaf9f0171e9517f40e41\"");
      response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
    } else {
      writeResponse(response);
    }
  }

  // a simple implementation of authentication for Digest, Basic and NTLM
  private boolean isAuthenticated(HttpServletRequest request) {

    String authHeader = request.getHeader("Authorization");
    if (authHeader != null && authHeader.startsWith("Digest ")) {
      String[] params = authHeader.substring("Digest ".length()).split(",\\s*");
      String username = Arrays.stream(params).filter(param -> param.startsWith("username=")).findFirst()
          .map(param -> param.split("=")[1].replace("\"", "").trim()).orElse(null);
      return username.equals("testUsername");
    } else if (authHeader != null && authHeader.startsWith("NTLM")) {
      return true;
    } else if (authHeader != null && authHeader.startsWith("Basic ")) {
      String base64Credentials = authHeader.substring("Basic ".length()).trim();
      String credentials = new String(Base64.getDecoder().decode(base64Credentials));
      final String[] values = credentials.split(":", 2);
      String username = values[0];
      String password = values[1];
      return "testUsername".equals(username) && "testPassword".equals(password);
    }
    return false;
  }


  protected void extractBaseRequestParts(Request baseRequest) throws IOException {
    method = baseRequest.getMethod();
    uri = baseRequest.getHttpURI().getPathQuery();

    extractHeadersFromBaseRequest(baseRequest);

    HttpInputOverHTTP inputStream = (HttpInputOverHTTP) baseRequest.getInputStream();
    StringBuffer buffer = new StringBuffer();
    while (!inputStream.isFinished()) {
      int toRead = inputStream.available();
      byte[] bytes = new byte[toRead];
      int bytesRead = inputStream.read(bytes);
      if (bytesRead > 0) {
        buffer.append(new String(bytes));
      }
    }
    body = buffer.toString();
  }

  protected void extractHeadersFromBaseRequest(Request baseRequest) {
    for (String headerName : getHeaderNames(baseRequest)) {
      Enumeration<String> headerValues = baseRequest.getHeaders(headerName);

      while (headerValues.hasMoreElements()) {
        headers.put(headerName, headerValues.nextElement());
      }
    }
  }

  protected void writeResponse(HttpServletResponse response) throws IOException {
    response.setContentType("application/json");
    response.setStatus(HttpServletResponse.SC_OK);
    response.getWriter().print(DEFAULT_RESPONSE);
  }

  private List<String> getHeaderNames(Request baseRequest) {
    Enumeration<String> headerNames = baseRequest.getHeaderNames();
    List<String> headerList = new ArrayList<>();
    while (headerNames.hasMoreElements()) {
      headerList.add(headerNames.nextElement());
    }
    return headerList;
  }

  public String getRequestBody() {
    return body;
  }
}

==> mule-netty-http-service-0.3.2-20251127.174835-27-test-sources.jar: org/mule/service/http/test/netty/utils/server/AuthenticationTestServer.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 */
package org.mule.service.http.test.netty.utils.server;

import static com.google.common.collect.Multimaps.newMultimap;

import org.mule.runtime.api.util.CaseInsensitiveMapWrapper;
import org.mule.service.http.test.netty.utils.NtlmLoginAuthenticator;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Base64;
import java.util.Enumeration;
import java.util.List;

import com.google.common.collect.Multimap;
import com.google.common.collect.Sets;

import org.eclipse.jetty.security.ConstraintMapping;
import org.eclipse.jetty.security.ConstraintSecurityHandler;
import org.eclipse.jetty.security.HashLoginService;
import org.eclipse.jetty.security.LoginService;
import org.eclipse.jetty.security.authentication.BasicAuthenticator;
import org.eclipse.jetty.security.authentication.DigestAuthenticator;
import org.eclipse.jetty.server.Handler;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.handler.AbstractHandler;
import org.eclipse.jetty.server.handler.ContextHandler;
import org.eclipse.jetty.server.handler.ContextHandlerCollection;
import org.eclipse.jetty.util.security.Constraint;
import org.junit.rules.ExternalResource;

import jakarta.servlet.ServletException;
import jakarta.servlet.ServletInputStream;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

public class AuthenticationTestServer extends ExternalResource {

  public int httpPort;
  public static final String DEFAULT_RESPONSE = "{\"key1\":\"value1\",\"key2\":\"value2\"}";
  protected volatile Server server;

  protected volatile String method;
  protected volatile String uri;
  protected volatile Multimap<String, String> headers = newMultimap(new CaseInsensitiveMapWrapper<>(), Sets::newHashSet);

  protected volatile String body;

  public AuthenticationTestServer(int port) {
    this.httpPort = port;
  }

  @Override
  public void before() throws Exception {
    server = createServer();
    AbstractHandler handler = new TestHandler();
    String realmPath = new File(this.getClass().getClassLoader().getResource("auth/realm.properties").toURI()).getAbsolutePath();
    server.setHandler(createAuthHandler(server, handler, realmPath));
    server.start();
  }

  private AbstractHandler createAuthHandler(Server server, AbstractHandler handler, String realmPath) {
    LoginService loginService = new HashLoginService("TestRealm", realmPath);
    server.addBean(loginService);

    ContextHandler basicContext = createBasicContextHandler(handler);
    ContextHandler digestContext = createDigestContextHandler(handler);
    ContextHandler ntlmContext = createNtlmContextHandler(handler);
    ContextHandler multiAuthContext = createMultiAuthContextHandler(handler);

    ContextHandlerCollection handlers = new ContextHandlerCollection();
    handlers.setHandlers(new Handler[] {basicContext, digestContext, ntlmContext, multiAuthContext});
    return handlers;
  }

  private ContextHandler createDigestContextHandler(AbstractHandler handler) {
    Constraint digestConstraint = new Constraint();
    digestConstraint.setName("auth");
    digestConstraint.setRoles(new String[] {"user"});
    digestConstraint.setAuthenticate(true);

    ConstraintMapping digestConstraintMapping = new ConstraintMapping();
    digestConstraintMapping.setConstraint(digestConstraint);
    digestConstraintMapping.setPathSpec("/*");

    ConstraintSecurityHandler digestSecurityHandler = new ConstraintSecurityHandler();
    digestSecurityHandler.setAuthenticator(new DigestAuthenticator());
    digestSecurityHandler.setConstraintMappings(new ConstraintMapping[] {digestConstraintMapping});
    digestSecurityHandler.setHandler(handler);

    ContextHandler digestContext = new ContextHandler("/digest");
    digestContext.setHandler(digestSecurityHandler);

    return digestContext;
  }

  protected ContextHandler createBasicContextHandler(AbstractHandler handler) {
    Constraint basicConstraint = new Constraint();
    basicConstraint.setName("auth");
    basicConstraint.setRoles(new String[] {"user"});
    basicConstraint.setAuthenticate(true);

    ConstraintMapping basicConstraintMapping = new ConstraintMapping();
    basicConstraintMapping.setConstraint(basicConstraint);
    basicConstraintMapping.setPathSpec("/*");

    ConstraintSecurityHandler basicSecurityHandler = new ConstraintSecurityHandler();
    basicSecurityHandler.setAuthenticator(new BasicAuthenticator());
    basicSecurityHandler.setConstraintMappings(new ConstraintMapping[] {basicConstraintMapping});
    basicSecurityHandler.setHandler(handler);

    ContextHandler basicContext = new ContextHandler("/basic");
    basicContext.setHandler(basicSecurityHandler);

    return basicContext;
  }

  private ContextHandler createNtlmContextHandler(AbstractHandler handler) {
    Constraint ntlmConstraint = new Constraint();
    ntlmConstraint.setName("auth");
    ntlmConstraint.setRoles(new String[] {"user"});
    ntlmConstraint.setAuthenticate(true);

    ConstraintMapping ntlmConstraintMapping = new ConstraintMapping();
    ntlmConstraintMapping.setConstraint(ntlmConstraint);
    ntlmConstraintMapping.setPathSpec("/*");

    ConstraintSecurityHandler ntlmSecurityHandler = new ConstraintSecurityHandler();
    ntlmSecurityHandler.setAuthenticator(new NtlmLoginAuthenticator());
    ntlmSecurityHandler.setConstraintMappings(new ConstraintMapping[] {ntlmConstraintMapping});
    ntlmSecurityHandler.setHandler(handler);

    ContextHandler ntlmContext = new ContextHandler("/ntlm");
    ntlmContext.setHandler(ntlmSecurityHandler);

    return ntlmContext;
  }

  private ContextHandler createMultiAuthContextHandler(AbstractHandler handler) {

    Constraint basicConstraint = new Constraint();
    basicConstraint.setName("basicAuth");
    basicConstraint.setRoles(new String[] {"user"});
    basicConstraint.setAuthenticate(true);

    ConstraintMapping basicConstraintMapping = new ConstraintMapping();
    basicConstraintMapping.setConstraint(basicConstraint);
    basicConstraintMapping.setPathSpec("/multi-auth/*");

    ConstraintSecurityHandler basicSecurityHandler = new ConstraintSecurityHandler();
    basicSecurityHandler.setAuthenticator(new BasicAuthenticator());
    basicSecurityHandler.setConstraintMappings(new ConstraintMapping[] {basicConstraintMapping});

    Constraint digestConstraint = new Constraint();
    digestConstraint.setName("digestAuth");
    digestConstraint.setRoles(new String[] {"user"});
    digestConstraint.setAuthenticate(true);

    ConstraintMapping digestConstraintMapping = new ConstraintMapping();
    digestConstraintMapping.setConstraint(digestConstraint);
    digestConstraintMapping.setPathSpec("/multi-auth/*");

    ConstraintSecurityHandler digestSecurityHandler = new ConstraintSecurityHandler();
    digestSecurityHandler.setAuthenticator(new DigestAuthenticator());
    digestSecurityHandler.setConstraintMappings(new ConstraintMapping[] {digestConstraintMapping});

    Constraint ntlmConstraint = new Constraint();
    ntlmConstraint.setName("ntlmAuth");
    ntlmConstraint.setRoles(new String[] {"user"});
    ntlmConstraint.setAuthenticate(true);

    ConstraintMapping ntlmConstraintMapping = new ConstraintMapping();
    ntlmConstraintMapping.setConstraint(ntlmConstraint);
    ntlmConstraintMapping.setPathSpec("/multi-auth/*");

    ConstraintSecurityHandler ntlmSecurityHandler = new ConstraintSecurityHandler();
    ntlmSecurityHandler.setAuthenticator(new NtlmLoginAuthenticator());
    ntlmSecurityHandler.setConstraintMappings(new ConstraintMapping[] {ntlmConstraintMapping});

    ContextHandlerCollection securityHandlers = new ContextHandlerCollection();
    securityHandlers.setHandlers(new Handler[] {basicSecurityHandler, digestSecurityHandler, ntlmSecurityHandler});

    ContextHandler multiAuthContext = new ContextHandler("/multi-auth");
    multiAuthContext.setHandler(securityHandlers);

    multiAuthContext.setHandler(new ContextHandlerCollection() {

      @Override
      public void handle(String target, Request baseRequest, HttpServletRequest request, HttpServletResponse response)
          throws IOException, ServletException {
        securityHandlers.handle(target, baseRequest, request, response);
        if (!baseRequest.isHandled()) {
          handler.handle(target, baseRequest, request, response);
        }
      }
    });

    return multiAuthContext;
  }

  @Override
  public void after() {
    if (server != null) {
      try {
        server.stop();
      } catch (Exception e) {
        throw new RuntimeException(e);
      }
    }
  }

  protected Server createServer() {
    return new Server(httpPort);
  }

  public void addHandler(AbstractHandler handler) {
    this.server.setHandler(handler);
  }

  private class TestHandler extends AbstractHandler {

    @Override
    public void handle(String target, Request baseRequest, HttpServletRequest request, HttpServletResponse response)
        throws IOException {

      handleRequest(baseRequest, request, response);
      baseRequest.setHandled(true);
    }
  }

  protected void handleRequest(Request baseRequest, HttpServletRequest request, HttpServletResponse response) throws IOException {
    extractBaseRequestParts(baseRequest);

    if (!isAuthenticated(request)) {
      response.addHeader("WWW-Authenticate", "Basic realm=\"Test\"");
      response.addHeader("WWW-Authenticate", "NTLM");
      response
          .addHeader("WWW-Authenticate",
                     "Digest realm=\"Test\", qop=\"auth\", nonce=\"dcd98b7102dd2f0e8b11d0f600bfb0c093\", opaque=\"5ccc069c403ebaf9f0171e9517f40e41\"");
      response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
    } else {
      writeResponse(response);
    }
  }

  // a simple implementation of authentication for Digest, Basic and NTLM
  private boolean isAuthenticated(HttpServletRequest request) {

    String authHeader = request.getHeader("Authorization");
    if (authHeader != null && authHeader.startsWith("Digest ")) {
      String[] params = authHeader.substring("Digest ".length()).split(",\\s*");
      String username = Arrays.stream(params).filter(param -> param.startsWith("username=")).findFirst()
          .map(param -> param.split("=")[1].replace("\"", "").trim()).orElse(null);
      return username.equals("testUsername");
    } else if (authHeader != null && authHeader.startsWith("NTLM")) {
      return true;
    } else if (authHeader != null && authHeader.startsWith("Basic ")) {
      String base64Credentials = authHeader.substring("Basic ".length()).trim();
      String credentials = new String(Base64.getDecoder().decode(base64Credentials));
      final String[] values = credentials.split(":", 2);
      String username = values[0];
      String password = values[1];
      return "testUsername".equals(username) && "testPassword".equals(password);
    }
    return false;
  }


  protected void extractBaseRequestParts(Request baseRequest) throws IOException {
    method = baseRequest.getMethod();
    uri = baseRequest.getHttpURI().getPathQuery();

    extractHeadersFromBaseRequest(baseRequest);

    ServletInputStream inputStream = baseRequest.getInputStream();
    StringBuffer buffer = new StringBuffer();
    while (!inputStream.isFinished()) {
      int toRead = inputStream.available();
      byte[] bytes = new byte[toRead];
      int bytesRead = inputStream.read(bytes);
      if (bytesRead > 0) {
        buffer.append(new String(bytes));
      }
    }
    body = buffer.toString();
  }

  protected void extractHeadersFromBaseRequest(Request baseRequest) {
    for (String headerName : getHeaderNames(baseRequest)) {
      Enumeration<String> headerValues = baseRequest.getHeaders(headerName);

      while (headerValues.hasMoreElements()) {
        headers.put(headerName, headerValues.nextElement());
      }
    }
  }

  protected void writeResponse(HttpServletResponse response) throws IOException {
    response.setContentType("application/json");
    response.setStatus(HttpServletResponse.SC_OK);
    response.getWriter().print(DEFAULT_RESPONSE);
  }

  private List<String> getHeaderNames(Request baseRequest) {
    Enumeration<String> headerNames = baseRequest.getHeaderNames();
    List<String> headerList = new ArrayList<>();
    while (headerNames.hasMoreElements()) {
      headerList.add(headerNames.nextElement());
    }
    return headerList;
  }

  public String getRequestBody() {
    return body;
  }
}
